-----------------------------------------------------------------------------------------------
-- Client Lua Script for CitBand
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "ICCommLib"
require "ChatSystemLib"
-----------------------------------------------------------------------------------------------
-- CitBand Module Definition
-----------------------------------------------------------------------------------------------
local CitBand = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CitBand:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tItems = {} -- keep track of all the list items
    return o
end

function CitBand:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- CitBand OnLoad
-----------------------------------------------------------------------------------------------
function CitBand:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CitBand.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- CitBand OnDocLoaded
-----------------------------------------------------------------------------------------------
function CitBand:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CitBandForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndSettings = Apollo.LoadForm(self.xmlDoc, "CitBandSettingsForm", nil, self)
		if self.wndSettings == nil then
            Apollo.AddAddonErrorText(self, "Could not load the child window for some reason.")
            return
        end

		self.wndSend = self.wndMain:FindChild("etSend")
		self.wndRecv = self.wndMain:FindChild("etRecv")
		self.playerName = nil
		
	    self.wndMain:Show(false, true)
		self.wndSettings:Show(false, true)
		self.wndChName = {}
		self.chActive = {}
		for k = 0, 3, 1 do
			self.wndChName[k] = self.wndSettings:FindChild("etCh" .. (k+1))
			self.tItems[k] = {}
			if self.tSavedData ~= nil and self.tSavedData[k] ~= nil then
				self.tItems[k].chName = self.tSavedData[k]
				self.wndMain:FindChild("btnCh" .. (k+1)):SetText(self.tItems[k].chName)
			else
				self.tItems[k].chName = self.wndMain:FindChild("btnCh" .. (k+1)):GetText()
    		end
			self.wndChName[k]:SetText(self.tItems[k].chName)
			self.chActive[k] = ICCommLib.JoinChannel("cbr" .. self.tItems[k].chName, "OnMessageCh" .. (k+1), self)
	    end

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("cbr", "OnCitBandOn", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- CitBand Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/cbr"
function CitBand:OnCitBandOn()
	self.wndMain:Invoke() -- show the window
	--if self.tSavedData ~= nil and self.tSavedData[0] ~= nil then
	--	ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Command, self.tSavedData[0], "" )
	--end

end

function CitBand:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end

    -- create a table to hold our data
    local tSave = {}
    for k = 0, 3, 1 do
        tSave[k] = self.tItems[k].chName
    end

    -- simply return this value and Apollo will save the file!
    return tSave
end

function CitBand:OnRestore(eLevel, tData)
    -- just store this and use it later
    self.tSavedData = tData
end
-----------------------------------------------------------------------------------------------
function CitBand:OnMessage(channel, tMsg)
	local t = self.wndRecv:GetText()
	t = t .. "\n(" .. channel .. ") " .. tMsg.strPlayer .. "> " .. tMsg.strMessage
	self.wndRecv:SetText(t)
	self.wndRecv:SetVScrollPos(self.wndRecv:GetVScrollRange())
end

function CitBand:OnMessageCh1(channel, tMsg)
	self:OnMessage(self.tItems[0].chName, tMsg)
end

function CitBand:OnMessageCh2(channel, tMsg)
	self:OnMessage(self.tItems[1].chName, tMsg)
end

function CitBand:OnMessageCh3(channel, tMsg)
	self:OnMessage(self.tItems[2].chName, tMsg)
end

function CitBand:OnMessageCh4(channel, tMsg)
	self:OnMessage(self.tItems[3].chName, tMsg)
end
-----------------------------------------------------------------------------------------------
-- CitBandForm Functions
-----------------------------------------------------------------------------------------------

-- when the Cancel button is clicked
function CitBand:OnCancel()
	self.wndMain:Close() -- hide the window
end


function CitBand:OnClear( wndHandler, wndControl, eMouseButton )
	self.wndRecv:SetText("")
	self.wndSend:SetText("")
end

function CitBand:OnSettings( wndHandler, wndControl, eMouseButton )
	self.wndSettings:Invoke()
end

function CitBand:OnCh1()
	self:SendMessage(0)
end

function CitBand:OnCh2()
	self:SendMessage(1)
end

function CitBand:OnCh3()
	self:SendMessage(2)
end

function CitBand:OnCh4()
	self:SendMessage(3)
end

function CitBand:SendMessage(channel)
	local t = {}
	t.strMessage = self.wndSend:GetText()
	if t.strMessage == "" then
		return
	end
	if self.playerName == nil then
		if GameLib.GetPlayerUnit():GetName() ~= nil then
			self.playerName = GameLib.GetPlayerUnit():GetName()
		else
			return
		end
	end
	t.strPlayer = self.playerName
	self.chActive[channel]:SendMessage(t)
	self:OnMessage(self.tItems[channel].chName, t)
	self.wndSend:SetText("")

---------------------------------------------------------------------------------------------------
-- CitBandSettingsForm Functions
---------------------------------------------------------------------------------------------------
end
function CitBand:OnSaveSettings( wndHandler, wndControl, eMouseButton )
	for k = 0, 3, 1 do
		self.tItems[k].chName = self.wndChName[k]:GetText()
		self.wndMain:FindChild("btnCh" .. (k+1)):SetText(self.tItems[k].chName)
		self.chActive[k] = ICCommLib.JoinChannel("cbr" .. self.tItems[k].chName, "OnMessageCh" .. (k+1), self)
    end
	--self:OnSave(CodeEnumAddonSaveLevel.Account)
	self.wndSettings:Close()
end

function CitBand:OnCancelSettings( wndHandler, wndControl, eMouseButton )
	for k = 0, 3, 1 do
		self.wndChName[k]:SetText(self.tItems[k].chName)
    end
	self.wndSettings:Close()
end

-----------------------------------------------------------------------------------------------
-- CitBand Instance
-----------------------------------------------------------------------------------------------
local CitBandInst = CitBand:new()
CitBandInst:Init()
